@@ -1,7 +1,7 @@
 /*GPL*START*
  * mp3check - check mp3 file for consistency and print infos
  * 
- * Copyright (C) 1998-2002 by Johannes Overmann <overmann@iname.com>
+ * Copyright (C) 1998-2005 by Johannes Overmann <Johannes.Overmann@gmx.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include <sys/stat.h>
 #include <sys/mman.h>
+#include <sys/ioctl.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -33,14 +34,14 @@
 
 
 
-#define ONLY_MP3 "mp3,MP3"
+#define ONLY_MP3 "mp3,MP3,Mp3,mP3"
 
 
 const char *options[] = {
    "#usage='Usage: [OPTIONS, FILES AND DIRECTORIES] [--] [FILES AND DIRECTORIES]\n\n"
      "this program checks audio mpeg layer 1,2 and 3 (*.mp3) files for\n"       
      "consistency (headers and crc) and anomalies'",
-   "#trailer='\n%n version %v *** (C) 1998-2001 by Johannes Overmann\ncomments, bugs and suggestions welcome: %e\n%gpl'",
+   "#trailer='\n%n version %v *** (C) 1998-2003 by Johannes Overmann\ncomments, bugs and suggestions welcome: %e\n%gpl'",
    "#stopat--",
    "#onlycl",					 
    // options
@@ -63,9 +64,10 @@ const char *options[] = {
    "name=cut-tag-end      , type=switch,       , help='remove trailing tag'",
    "name=fix-headers      , type=switch,       , help='fix invalid headers (prevent constant parameter switching), implies -e, use with care'",
    "name=fix-crc          , type=switch,       , help='fix crc (set crc to the calculated one), implies -e, use with care\n(note: it is not possible to add crc to files which have been created without crc)'",
+   "name=add-tag          , type=switch,       , help='add ID3 v1.1 tag calculated from filename and path using simple heuristics if a file does not already have a tag'",
 
    "name=ign-tag128       , type=switch, char=G, help='ignore 128 byte TAG after last frame', headline='disable error messages for -e --error-check:'",
-   "name=ign-resync       , type=switch, char=Y, help='ignore invalid frame header'",
+   "name=ign-resync       , type=switch, char=Y, help='ignore synchronization errors (invalid frame header/frame too long/short)'",
    "name=ign-junk-end     , type=switch, char=E, help='ignore junk after last frame'",
    "name=ign-crc-error    , type=switch, char=Z, help='ignore crc errors'",
    "name=ign-non-ampeg    , type=switch, char=N, help='ignore non audio mpeg streams'",
@@ -101,6 +103,8 @@ const char *options[] = {
    "name=alt-color        , type=switch, char=b, help='colorize: do not use bold ANSI sequences'",
    "name=ascii-only       , type=switch,         help='replace the range of ASCII chars 160-255 (which is usually printable: e.g. ISO-8859) by \\'?\\''",
    "name=progress         , type=switch, char=p, help='show progress information on stderr'", 
+   "name=verbose          , type=switch, char=v, help='be more verbose'",
+   "name=no-mmap          , type=switch,       , help='do not use mmap (e.g. when you get \\'mmap: No such device\\')'",
    "name=dummy            , type=switch, char=0, help='do not write/modify anything other than the logfile', headline=common options:",
    "EOL" // end of list     
 };
@@ -135,8 +139,8 @@ const char *c_val = "\033[34m";
 const char *c_ok  = "\033[32m";
 const char *c_nor = "\033[0m";
 
-// minimum number of sequential valid and constant header to validate header
-const int min_valid = 6;
+// minimum number of sequential valid and constant frame headers to validate header
+const int MIN_VALID = 6;
 
 
 // global data
@@ -484,7 +488,7 @@ inline int find_next_header(const unsigned char *p, int len, int min_valid) {
 const unsigned char *skip_n_frames(const unsigned char *start, int len, int n) {
    while(1) {
       // skip invalid data
-      int s = find_next_header(start, len, min_valid);
+      int s = find_next_header(start, len, MIN_VALID);
       if(s < 0) return 0;
       start += s;
       len -= s;
@@ -541,7 +545,7 @@ void fmes(const char *name, const char *format, ...) {
 bool error_check(const char *name, const unsigned char *stream, int len, CRC16& crc, bool fix_headers, bool fix_crc) {
    int errors = 0;
    const unsigned char *p = stream;
-   int start = find_next_header(p, len, min_valid);
+   int start = find_next_header(p, len, MIN_VALID);
    int rest = len;
    int frame=0;
    double time=0.0;
@@ -636,7 +640,7 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
 	    // else look for a regular stream
       	    h = get_header(p+s);
       	    if(!head.sameConstant(h))
-	       s = find_next_header(p, rest, min_valid);
+	       s = find_next_header(p, rest, MIN_VALID);
 	    if(s<0) { // error: junk at eof
 	       p+=(l-4);
 	       start+=(l-4);
@@ -646,14 +650,14 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
       	    
 	    if(!ign_sync) {
 	       if(s<l-4) {
-		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sframe too short%s at %s0x%08x%s, %s%d%s byte%s mising\n", 
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %ssync error (frame too short)%s at %s0x%08x%s, %s%d%s byte%s mising\n", 
 		       cval, frame - 1, cnor,
 		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
 		       cerror, cnor, cval, start - 4, cnor,
 		       cval, l-4-s, cnor, (l-4-s>1)?"s":"");
 		  errors++;
 	       } else {
-		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %sframe too long%s at %s0x%08x%s, skipping %s%d%s byte%s at %s0x%08x%s\n", 
+		  fmes(name, "frame %s%5d%s/%s%2u:%02u%s: %ssync error (frame too long)%s at %s0x%08x%s, skipping %s%d%s byte%s at %s0x%08x%s\n",
 		       cval, frame - 1, cnor,
 		       cval, (unsigned int)(time/1000)/60, (unsigned int)(time/1000)%60, cnor,
 		       cerror, cnor, cval, start - 4, cnor,
@@ -929,7 +933,7 @@ bool error_check(const char *name, const unsigned char *stream, int len, CRC16&
 // returns true on anomaly
 bool anomaly_check(const char *name, const unsigned char *p, int len, bool err_check, int& err) {
    bool had_ano = false;
-   int start = find_next_header(p, len, min_valid);
+   int start = find_next_header(p, len, MIN_VALID);
    if(start>=0) {
       Header h = get_header(p+start);
       if(!ano_any_ver) {
@@ -994,7 +998,7 @@ bool anomaly_check(const char *name, const unsigned char *p, int len, bool err_c
 // returns the stream duration in ms
 // also returns minimum, maximum and average bitrates if told to
 unsigned int stream_duration(const unsigned char *p, int len, unsigned short int *minbr, unsigned short int *maxbr, unsigned short int *avgbr) {
-   int next = find_next_header(p, len, min_valid);
+   int next = find_next_header(p, len, MIN_VALID);
    int rest = len - next;
    double duration = 0.0;
    unsigned short int min = 2048, max = 0;
@@ -1006,7 +1010,7 @@ unsigned int stream_duration(const unsigned char *p, int len, unsigned short int
       Header h=get_header(p+next);
       if(!h.isValid()) {
       	 int old = next;
-	 next = find_next_header(p+old, rest, min_valid);
+	 next = find_next_header(p+old, rest, MIN_VALID);
 	 if(next<0) break;
 	 rest -= next;
 	 next += old;
@@ -1035,7 +1039,7 @@ unsigned int stream_duration(const unsigned char *p, int len, unsigned short int
 bool cut_junk_end(const char *name, const unsigned char *p, int len, const unsigned char *free_p, int fd, int& err) {
 // this is basically the error_check routine which treats only the trainling junk case
 // (implemented by Pollyanna Lindgren <jtlindgr@cs.helsinki.fi>)
-   int start = find_next_header(p, len, min_valid);
+   int start = find_next_header(p, len, MIN_VALID);
    int rest = len;
    int frame=0;
    int l,s;
@@ -1089,10 +1093,10 @@ bool cut_junk_end(const char *name, const unsigned char *p, int len, const unsig
 	    }
 	 }
 	 if(!(h.isValid()&&(frame_length(h)>=21))) {
-	    // invalid header 
+	    // invalid header
 	    
 	    // search for next valid header
-	    s = find_next_header(p, rest, min_valid);
+	    s = find_next_header(p, rest, MIN_VALID);
 	    
 	    if(s<0) break; // error: junk at eof
 	    
@@ -1170,7 +1174,7 @@ bool cut_junk_end(const char *name, const unsigned char *p, int len, const unsig
 bool cut_tag_end(const char *name, const unsigned char *p, int len, int fd, int& err) {
 // this is basically the cut_junk_end routine that only looks for a 128 bytes trailing tag
 // (implemented by Jean Delvare <delvare@ensicaen.ismra.fr>)
-   int start = find_next_header(p, len, min_valid);
+   int start = find_next_header(p, len, MIN_VALID);
    Tagv1* tag;
 
    if(start<0) {
@@ -1218,13 +1222,125 @@ bool cut_tag_end(const char *name, const unsigned char *p, int len, int fd, int&
    }
 }
 
+// check for ID3 v1.x tag
+bool checkForID3V1(const unsigned char *p, size_t len)
+{
+    if(len < 128)
+	return false;
+    return memcmp(p + len - 128, "TAG", 3) == 0;
+}
 
+// check for ID3 v2.x tag
+bool checkForID3V2(const unsigned char *p, size_t len)
+{
+    if(len < 10)
+	return false;
+    return (memcmp(p + len - 10, "3DI", 3) == 0) || (memcmp(p, "ID3", 3) == 0);
+}
+
+// check for plausible string
+bool isValidStr(const unsigned char *p, size_t len)
+{
+    for(size_t i = 0; (i < len) && (i < 30); i++)
+    {
+	if(p[i] == 0)
+	    return i > 2;
+	if(!isprint(p[i]))
+	    return false;
+    }
+	return true;
+}
+
+// split artist and title
+void splitArtistTitle(const tstring& title, tstring &artistOut, tstring &titleOut)
+{
+    tstring origtitle = title;
+    const char *start = title.c_str();
+    int patlen = 3;
+    const char *p = strstr(start, " - ");
+    if(p == 0)
+    {
+	patlen = 1;
+	p = strstr(start, "-");
+    }
+    if(p)
+    {
+	size_t len = p - start;
+	artistOut = title.substr(0, len);
+	titleOut = title.substr(len + patlen);
+	if((strcasecmp(titleOut.substr(0, 3).c_str(), "Vol") == 0) ||
+	   (strcasecmp(titleOut.substr(0, 3).c_str(), "CD1") == 0) ||
+	   (strcasecmp(titleOut.substr(0, 3).c_str(), "CD3") == 0) ||
+	   (strcasecmp(titleOut.substr(0, 3).c_str(), "CD4") == 0) ||
+	   (strcasecmp(titleOut.substr(0, 3).c_str(), "CD2") == 0))
+	{
+	    titleOut = origtitle;
+	    artistOut = "";
+	}
+	if(artistOut == "Various")
+	    artistOut = "";
+	if(artistOut == "Sampler")
+	    artistOut = "";
+    }
+    else
+    {
+	artistOut = "";
+	titleOut = origtitle;
+    }
+}
+
+
+// check for possible tags more sloppy
+bool checkForTagsSloppy(const unsigned char *p, size_t len)
+{
+    if(len < 10)
+	return false;
+    size_t max = 1024*1024*1024;
+    if(max > len - 3)
+	max = len - 3;
+    for(size_t i = 0; i <= max; i++)
+    {
+	if(
+	   ((memcmp(p + i, "TAG", 3) == 0) && (isValidStr(p + i + 3, len - i - 3))) ||
+	   (
+	    (
+	     (memcmp(p + i, "ID3", 3) == 0) ||
+	     (memcmp(p + i, "3DI", 3) == 0)
+	     ) && 
+	    (p[i + 3] < 10) && (p[i + 4] < 10) && ((p[i + 5] & 0xf) == 0) && 
+	    (p[i + 6] < 128) && (p[i + 7] < 128) && (p[i + 8] < 128) && (p[i + 9] < 128)))
+	{
+	    printf("%swarning: possible %c%c%c tag found at offset %d (%d from end) (%02x %02x %02x %02x %02x %02x %02x)%s\n", 
+		   cano, p[i+0], p[i+1], p[i+2], int(i), int(len - 3 - i), p[i+3]&255, p[i+4]&255, p[i+5]&255, 
+		   p[i+6]&255, p[i+7]&255, p[i+8]&255, p[i+9]&255, cnor);
+	    return true;
+	}
+    }
+    return false;
+}
+
+// capitalize string
+void capitalize(tstring &str)
+{
+    bool lastAlpha = false;
+    for(size_t i = 0; i < str.length(); i++)
+    {
+	if((!lastAlpha) && islower(str[i]))
+	    str[i] = toupper(str[i]);
+	lastAlpha = isalpha(str[i]);
+    }
+}
+
+// main
 int main(int argc, char *argv[]) {      
 
    // get parameters
    TAppConfig ac(options, "options", argc, argv, 0, 0, VERSION);
    
    // get the terminal width if available
+   struct winsize win;
+   if(ioctl(1, TIOCGWINSZ, &win) == 0) 
+       columns = win.ws_col;
    const char* scolumns=getenv("COLUMNS");
    if(scolumns)
       columns=atoi(scolumns)-1;
@@ -1237,11 +1353,13 @@ int main(int argc, char *argv[]) {
    progress = ac("progress");
    max_errors = ac.getInt("max-errors");
    show_valid_files = ac("show-valid");
+   bool nommap = ac("no-mmap");
    int opt=0;
    // alt mode
    if(ac("error-check")) opt=1; 
    if(ac("fix-headers")) opt=1;
    if(ac("fix-crc")) opt=1;
+   if(ac("add-tag")) opt=1;
    if(ac("anomaly-check")) opt=1; 
    if(ac("cut-junk-start")) opt=1; 
    if(ac("cut-junk-end")) opt=1;
@@ -1336,7 +1454,7 @@ int main(int argc, char *argv[]) {
 	    TFile f(ac.param(i));
 	    if(f.isdir()) {
 	       TSubTreeContext context(cross_filesystems);
-	       TDir d(f, &context);
+	       TDir d(f, context);
 	       filelist += findFilesRecursive(d);
 	       continue;
 	    }
@@ -1372,6 +1490,7 @@ int main(int argc, char *argv[]) {
    int err=0;
    int checked=0;
    int num_ano=0;
+   int num_tagsadded = 0;
    CRC16 crc(CRC16::CRC_16);   
    FILE *log = NULL;
    if(!ac.getString("log-file").empty()) {
@@ -1380,9 +1499,16 @@ int main(int argc, char *argv[]) {
 	userError("can't open logfile '%s'!\n", ac.getString("log-file").c_str());
    }
    for(size_t i = 0; i < filelist.size(); i++) {
-      
-      // check for file
       const char *name = filelist[i].c_str();
+      // ignore all files starting with ._ which are apple metafiles
+      {
+	  tstring t = name;
+	  t.extractFilename();
+	  if((t[0] == '.') && (t[1] == '_'))
+	      continue;
+      }
+       
+      // check for file
       struct stat buf;
       if(stat(name, &buf)) {
 	 fmes(name, "%scan't stat file (dangling symbolic link?)%s\n", cerror, cnor);
@@ -1405,6 +1531,8 @@ int main(int argc, char *argv[]) {
 	 if(ac("fix-headers")||ac("cut-junk-start")||ac("fix-crc")||ac("cut-junk-end")||ac("cut-tag-end")||edit_frame_byte) {
 	    flags = O_RDWR;
 	    prot |= PROT_WRITE;
+	    if(nommap)
+		 userError("option --no-mmap does not yet support options which may modify a file (e.g --fix-crc)!\n");
 	 }
       }
       flags |= O_BINARY;
@@ -1413,17 +1541,28 @@ int main(int argc, char *argv[]) {
 	 perror("open");
 	 userError("can't open file '%s' for reading!\n", name);      
       }
-      // mmap file
+       
+      // mmap or read file
       const unsigned char *p;
-      if(len) {
-	 p = (const unsigned char *) mmap(0, len, prot, MAP_SHARED, fd, 0);
+      const unsigned char *free_p = 0;
+      if(nommap) {
+	  // read file
+	  free_p = p = new unsigned char[len];
+	  if(read(fd, (void*)p, len) != len) {
+	      perror("read");
+	      userError("error while reading file '%s'!\n", name);
+	  }
       } else {
-	 p = NULL;
-      }
-      const unsigned char *free_p = p;
-      if(p==(const unsigned char *)MAP_FAILED) {
-	 perror("mmap");
-	 userError("can't map file '%s'!\n", name);
+	  // mmap file
+	  if(len) {
+	      free_p = p = (const unsigned char *) mmap(0, len, prot, MAP_SHARED, fd, 0);
+	  } else {
+	      p = NULL;
+	  }
+	  if(p==(const unsigned char *)MAP_FAILED) {
+	      perror("mmap");
+	      userError("can't map file '%s'!\n", name);
+	  }
       }
 
       // edit single byte of a frame
@@ -1433,9 +1572,9 @@ int main(int argc, char *argv[]) {
 	    fprintf(stderr, "%-79.79s\r", s.c_str());
 	    fflush(stderr);
 	 }	 
-	 unsigned char *p = const_cast<unsigned char *>(skip_n_frames(free_p, len, efb_frame));
-	 if(p) {
-	    if(!dummy) p[efb_offset] = efb_value;
+	 unsigned char *pp = const_cast<unsigned char *>(skip_n_frames(free_p, len, efb_frame));
+	 if(pp) {
+	    if(!dummy) pp[efb_offset] = efb_value;
 	 } else {
 	    fmes(name, "%sframe %s%d%s not found%s\n", cerror, cval, efb_frame, cerror, cnor);
 	    err++;
@@ -1446,7 +1585,7 @@ int main(int argc, char *argv[]) {
       if(ac("list")||ac("compact-list")||ac("raw-list")) {
 	 // speed up list of very large files (like *.wav)
 	 int maxl = 128*1024; // search max 128k
-	 int start = find_next_header(p, len<maxl?len:maxl, min_valid);
+	 int start = find_next_header(p, len<maxl?len:maxl, MIN_VALID);
 	 if(start<0) {
 	    if(!ign_noamp) {
 	       if(ac("raw-list")) {
@@ -1555,7 +1694,7 @@ int main(int argc, char *argv[]) {
       
       // cut-junk-start
       if(ac("cut-junk-start")) {
-	 int start = find_next_header(p, len, min_valid);
+	 int start = find_next_header(p, len, MIN_VALID);
 	 if(start<0) {
 	    fmes(name, "%s%s%s\n", cerror, (len?"not an audio mpeg stream":"empty file"), cnor);
 	    err++;
@@ -1696,24 +1835,293 @@ int main(int argc, char *argv[]) {
 	 delete tag;
          if(err_thisfile) ++err;
       }
+       
+       // --add-id3
+       bool addTag = false;
+       if(ac("add-tag"))
+       {
+	   if(progress) {
+	       tstring s = tstring(name).shortFilename(79);
+	       fprintf(stderr, "%-79.79s\r", s.c_str());
+	       fflush(stderr);
+	   }
+	   // check for existing tag
+	   if(checkForID3V1(p, len))
+	   {
+	       if(ac("verbose"))
+		   fmes(name, "id3 tag v1.x found, not adding anything\n");
+	   }
+	   else if(checkForID3V2(p, len))
+	   {
+	       if(ac("verbose"))
+		   fmes(name, "id3 tag v2.x found, not adding anything\n");
+	   }
+#if 0	   
+	   // this wqs just here to make sure we do not mis something
+	   else if(checkForTagsSloppy(p, len))
+	   {
+	       fmes(name, "some tag found\n");	       
+	   }
+#endif	   
+	   else
+	   {
+	       // no tag found: add tag
+	       addTag = true;
+	   }
+       }
       
-      
-      // unmap file and close
-      if((free_p!=NULL)&&munmap((char*)free_p, len)) {
-	 perror("munmap");
-	 userError("can't unmap file '%s'!\n", name);
+      if(nommap) {
+	  // free mem
+	  delete[] free_p;
+      } else {
+	  // unmap file and close
+	  if((free_p!=NULL)&&munmap((char*)free_p, len)) {
+	      perror("munmap");
+	      userError("can't unmap file '%s'!\n", name);
+	  }
       }
-      close(fd);      
+       // close file
+      close(fd);       
+       
+       // add tag? (see above)
+       if(addTag)
+       {	   
+	   // extract data from filename
+	   tstring title;  // 30
+	   tstring artist; // 30
+	   tstring album;  // 30
+	   tstring comment;// 28
+	   char track = 0;     // 1
+	   tstring fname = name;
+	   fname.translateChar('_', ' ');
+	   fname.searchReplace("cd 1", "cd1");
+	   fname.searchReplace("cd 2", "cd2");
+	   fname.searchReplace("cd 3", "cd3");
+	   fname.searchReplace("cd 4", "cd4");
+	   fname.searchReplace(" - ms/disc1/", "/cd1 - ");
+	   fname.searchReplace(" - ms/disc2/", "/cd2 - ");
+	   fname.searchReplace("/cd1/", " - cd1/");
+	   fname.searchReplace("/cd2/", " - cd2/");
+	   fname.searchReplace("/cd3/", " - cd3/");
+	   fname.searchReplace("/cd4/", " - cd4/");
+	   fname.searchReplace("zance - a decade of dance from ztt", "zance decade of dance from ztt");
+	   fname.searchReplace("/captain future soundtrack - ", "/");
+	   fname.searchReplace("-ms/", "/");
+	   fname.collapseSpace();
+	   title = fname;
+	   album = fname;
+	   comment = fname;
+	   title.extractFilename();
+	   album.extractPath();
+	   album.removeDirSlash();
+	   album.extractFilename();
+	   comment.extractPath();
+	   comment.removeDirSlash();
+	   comment.extractPath();
+	   comment.removeDirSlash();
+	   comment.extractFilename();
+
+	   // title
+	   // remove album
+	   if(title != album)
+	       title.searchReplace(album, "");
+	   // remove .mp3
+	   if((title.length() >= 4) && strcasecmp(title.c_str() + title.length(), ".mp3"))
+	       title.truncate(title.length() - 4);
+	   // check for cd1 - 
+	   if((strcasecmp(title.substr(0, 3).c_str(), "CD1") == 0) ||
+	      (strcasecmp(title.substr(0, 3).c_str(), "CD1") == 0) ||
+	      (strcasecmp(title.substr(0, 3).c_str(), "CD2") == 0))
+	   {
+	       album += " " + title.substr(0,3);
+	       title = title.substr(3);
+	       // skip separator
+	       while(strchr(" -.", title.c_str()[0])) 
+		   title = title.substr(1);
+	   }
+	   // check for AA-TT format
+	   if(isdigit(title[0]) && isdigit(title[1]) && isdigit(title[3]) && isdigit(title[4]) && (title[2] == '-'))
+	   {
+	       album += " cd" + title.substr(1,2);
+	       title = title.substr(3);
+	   }
+	   // check for 'audio '
+	   tstring ltitle = title;
+	   ltitle.lower();
+ 	   if(ltitle.substr(0, 5) == "audio")
+	       title = title.substr(5) + " " + title.substr(0, 5);
+ 	   else if(ltitle.substr(0, 10) == "track cd -")
+	   {
+	       title = title.substr(10);
+	       album += " track cd";
+	   }
+ 	   else if(ltitle.substr(0, 8) == "mix cd -")
+	   {
+	       title = title.substr(8);
+	       album += " mix cd";
+	   }
+ 	   else if(ltitle.substr(0, 6) == "track-")
+	       title = title.substr(6) + " " + title.substr(0, 6);
+ 	   else if(ltitle.substr(0, 5) == "track")
+	       title = title.substr(5) + " " + title.substr(0, 5);
+	   title.cropSpace();
+	   title.collapseSpace();
+	   
+	   // scan track
+	   size_t pos = 0;
+	   while(isdigit(title.c_str()[pos])) pos++;
+	   if((pos >= 1) && (pos <=2))
+	   {
+	       int t = 0;
+	       title.substr(0, pos).toInt(t, 10);
+	       track = t;	       
+	   }
+	   else if(pos > 2)
+	       pos = 0;
+	   // skip separator
+	   while(strchr(" -.", title.c_str()[pos])) pos++;
+	   title = title.substr(pos);
+	   // split artist - title
+	   splitArtistTitle(title, artist, title);
+	   title.cropSpace();
+
+	   // album
+	   tstring ar;
+	   // split artist - album
+	   if((album == "alben") ||
+	      (album == "cdrom2") ||
+	      (album == "cdrom") ||
+	      (album == "misc1") ||
+	      (album == "mp3") ||
+	      (album == "ov") ||
+	      (album == "all"))
+	       album = "";
+	   splitArtistTitle(album, ar, album);
+	   if((!artist.empty()) && (!ar.empty()))
+	   {
+	       if(artist != ar)
+	       {
+		   title = artist + "-" + title;
+		   artist = ar;
+	       }
+	   }
+	   else if(artist.empty())
+	   {
+	       artist = ar;
+	       ar = "";
+	   }
+	   if(artist.empty())
+	   {
+	       artist = album;
+	       album = "";
+	   }
+	   album.cropSpace();
+	   artist.cropSpace();
+	   if(artist == "diverse")
+	       artist = "";
+	   
+	   // comment
+	   comment.cropSpace();
+	   if((comment == "alben") ||
+	      (comment == "cdrom2") ||
+	      (comment == "cdrom") ||
+	      (comment == "mp3") ||
+	      (comment == "ov") ||
+	      (comment == "chr music") ||
+	      (comment == "mp3 dl") ||
+	      (comment == "diverse") ||
+	      (comment.substr(0, 2) == "M0") ||
+	      (comment.substr(0, 7) == "Unknown") ||
+	      (comment.substr(0, 7) == "Various") ||
+	      (comment == "all"))
+	       comment = "";
+
+	   // capitalize strings
+	   capitalize(title);
+	   capitalize(artist);
+	   capitalize(album);
+	   capitalize(comment);
+	   
+	   // move rest of long strings into comment
+	   if(title.length() > 30)
+	       title.searchReplace(" - ", "-");
+	   if(artist.length() > 30)
+	       artist.searchReplace(" - ", "-");
+	   if(album.length() > 30)
+	       album.searchReplace(" - ", "-");
+	   if(title.length() > 30)
+	   {
+	       comment = "T:" + title.substr(30);
+	       title.truncate(30);
+	   }
+	   if(artist.length() > 30)
+	   {
+	       comment = "R:" + artist.substr(30);
+	       artist.truncate(30);
+	   }
+	   if(album.length() > 30)
+	   {
+	       comment = "L:" + album.substr(30);
+	       album.truncate(30);
+	   }
+
+	   // print and check tag
+//	   fmes(name, "appending id3 tag v1.1:\ntitle=  '%s%s%s'\nartist= '%s%s%s'\nalbum=  '%s%s%s'\ncomment='%s%s%s'\ntrack=  %s%d%s\n", 
+//		cval, title.c_str(), cnor, cval, artist.c_str(), cnor, cval, album.c_str(), cnor, cval, comment.c_str(), cnor, cval, track, cnor);
+	   fmes(name, "'%s%-30.30s%s' '%s%-30.30s%s' '%s%-30.30s%s' '%s%-28.28s%s' %s%d%s\n",
+		cval, title.c_str(), cnor, cval, artist.c_str(), cnor, cval, album.c_str(), cnor, cval, comment.c_str(), cnor, cval, track, cnor);
+	   if(title.length() > 30)
+	       printf("%swarning%s: title > 30 chars\n", cerror, cnor);
+	   if(artist.length() > 30)
+	       printf("%swarning%s: artist > 30 chars\n", cerror, cnor);
+	   if(album.length() > 30)
+	       printf("%swarning%s: album > 30 chars\n", cerror, cnor);
+	   if(comment.length() > 28)
+	       printf("%swarning%s: comment > 28\n", cerror, cnor);
+	   if(track == 0)
+	       printf("%swarning%s: no track\n", cerror, cnor);
+	   
+	   // write tag
+	   char tag[128];
+	   memset(tag, 0, 128);
+	   tag[0] = 'T';
+	   tag[1] = 'A';
+	   tag[2] = 'G';
+	   strncpy(tag + 3, title.c_str(), 30);
+	   strncpy(tag + 3 + 30, artist.c_str(), 30);
+	   strncpy(tag + 3 + 60, album.c_str(), 30);
+	   strncpy(tag + 3 + 90 + 4, comment.c_str(), 28);
+	   tag[126] = track;
+	   tag[127] = -1;
+	   num_tagsadded++;
+#if 1
+	   if(!dummy)
+	   {
+	       // append to file
+	       FILE *f = fopen(name, "ab");
+	       if(f == 0)
+		   userError("can't open file '%s' for writing!\n", name);
+	       if(fwrite(tag, 1, 128, f) != 128)
+		   userError("error while writing to file '%s'\n", name);
+	       fclose(f);
+	   }
+#endif
+       }
+       
       ++checked;
    } // for all params
 
    // print final statistics
    if((filelist.size()>1)&&(!ac("raw-list")) && (!ac("no-summary")) && (!quiet)) {
-      printf("--                                                                             \n%s%d%s file%s %s, %s%d%s erroneous file%s found\n", 
+      printf("--                                                                             \n"
+	     "%s%d%s file%s %s, %s%d%s erroneous file%s found\n", 
 	     cval, checked, cnor, checked==1?"":"s", 
 	     ac("list")?"listed":"checked", cval, err, cnor, err==1?"":"s");
       if(num_ano) printf("(%s%d%s %sanomal%s%s found)\n", 
-			 cval, num_ano, cnor, cano, num_ano==1?"y":"ies", cnor);      
+			 cval, num_ano, cnor, cano, num_ano==1?"y":"ies", cnor);
+       if(num_tagsadded)
+	   printf("(%s%d%s tags added)\n", 
+		  cval, num_tagsadded, cnor);
    }
    
    // end
