@@ -2,7 +2,7 @@
  *
  * tappconfig.cc - console application framework
  * 
- * Copyright (C) 1998 by Johannes Overmann <overmann@iname.com>
+ * Copyright (C) 1998 by Johannes Overmann <Johannes.Overmann@gmx.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 
 #include <ctype.h>
 #include <limits.h>
-#include <values.h>
+#include <float.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <sys/ioctl.h>
@@ -35,7 +35,7 @@
 // global email of author:
 const char *TAPPCONFIG_EMAIL = "Johannes.Overmann@gmx.de";
 // gpl message
-const char *TAPPCONFIG_GPL = "this program is distributed under the terms of the GNU General Public License";
+const char *TAPPCONFIG_GPL = "this program is distributed under the terms of the GNU General Public License version 2";
 
 
 // usage:
@@ -184,6 +184,10 @@ const char *options[] = {
 // 01:35 Jul 08 misc bugfixes
 // 2001: 
 // 22:58 Jun 11 userErrorExitStatus added (273/1946)
+// 2007:
+//       Jan 28 wasSetByUser() added
+//       Jan 29 print warning for unknown rc-file parameters instead of error (1977 lines)
+
 
 // global data:
 
@@ -486,11 +490,11 @@ static tstring Range2Str(int lower, int upper) {
 }
 
 static tstring Range2Str(double lower, double upper) {
-   if((lower!=-MAXDOUBLE) || (upper!=MAXDOUBLE)) {
+   if((lower!=-DBL_MAX) || (upper!=DBL_MAX)) {
       tstring lstr;
       tstring ustr;
-      if(lower != -MAXDOUBLE) lstr = tstring(lower);
-      if(upper !=  MAXDOUBLE) ustr = tstring(upper);
+      if(lower != -DBL_MAX) lstr = tstring(lower);
+      if(upper !=  DBL_MAX) ustr = tstring(upper);
       return "[" + lstr + ".." + ustr + "]";
    } else {
       return "";
@@ -585,12 +589,12 @@ void TAppConfigItem::validate(const char *line_context) {
 	 if(!upper.toDouble(double_upper))
 	   fatalError2("%s: illegal chars for upper bound in '%s'!\n", line_context, upper.c_str());
       }
-      else double_upper = MAXDOUBLE;
+      else double_upper = DBL_MAX;
       if(!lower.empty()) {
 	 if(!lower.toDouble(double_lower)) 
 	   fatalError2("%s: illegal chars for lower bound in '%s'!\n", line_context, lower.c_str());
       }
-      else double_lower = -MAXDOUBLE;
+      else double_lower = -DBL_MAX;
       if(tOutOfRange(double_default, double_lower, double_upper)) 
 	fatalError3("%s: default value out of range! (%g not in %s)!\n", line_context, double_default, Range2Str(double_lower, double_upper).c_str());
       break;
@@ -1210,10 +1214,10 @@ commonhead()
    if(verbose_conf || getBool("help") || getBool("hhelp") || getBool("version") || (!creatercfile.empty()))
      exit(0);
    for(size_t i = 0; i < opt.size(); i++) {
-      if(opt[i].must_have && (opt[i].set_in==DEFAULT))
+      if(opt[i].must_have && (opt[i].set_in==TAppConfigItem::DEFAULT))
 	userError("error: option '%s' must be specified somewhere!\n", 
 		  opt[i].name.c_str());
-      if(opt[i].should_have && (opt[i].set_in==DEFAULT))
+      if(opt[i].should_have && (opt[i].set_in==TAppConfigItem::DEFAULT))
 	userWarning("warning: option '%s' should be specified somewhere\n",
 	       opt[i].name.c_str());
    }
@@ -1232,15 +1236,17 @@ TAppConfig::~TAppConfig() {
 
 // save some items
 // return rcname in rc_name_out
-bool TAppConfig::save(tstring& rc_name_out) {
-   rc_name_out = "$(HOME)/." + rc_str;
+bool TAppConfig::save(tstring *rc_name_out) {
+   if(rc_name_out)
+	*rc_name_out = "$(HOME)/." + rc_str;
    char *home = getenv("HOME");
    if(home==0) {
       fprintf(stderr, "can't save config: no HOME variable defined!\n");
       return false;
    }
    tstring realname(tstring(home) + "/." + rc_str);
-   rc_name_out = realname;
+   if(rc_name_out)
+	*rc_name_out = realname;
    tstring tmpname(realname + ".tmp");
    FILE *tmpf = fopen(tmpname.c_str(), "w");
    if(tmpf==0) {
@@ -1372,16 +1378,26 @@ void TAppConfig::createRCFile(const tstring& fname, const tstring& rcname) const
 
 
 tstring TAppConfig::getName(const tstring& str, const tstring& context1, 
-			   const tstring& optprefix) const {
+			   const tstring& optprefix, bool errorIfNotFound) const {
    tstring context(context1);
    if(!context.empty()) context += ": ";
    if(str.len()==0) 
      fatalError1("%sempty option name!\n", context.c_str());
    if(alias.contains(str)) return opt[alias[str]].name; // alias match
    if((str.len()==1)&&(optprefix!="--")) { // char name
-      if(char2index[str[0]] < 0)
-	userError("%sunknown char option name '%s'! (try --help)\n", context.c_str(), str.c_str());
-      return opt[char2index[str[0]]].name;
+      if(char2index[(unsigned char)str[0]] < 0)
+      {
+       if (errorIfNotFound)
+       {
+	   userError("%sunknown char option name '%s'! (try --help)\n", context.c_str(), str.c_str());
+       }
+       else
+       {
+	   userWarning("%sunknown char option name '%s'!\n", context.c_str(), str.c_str());
+	   return "";
+       }
+      }
+      return opt[char2index[(unsigned char)str[0]]].name;
    }     
    if(name.contains(str)) return str; // exact match
    tvector<tstring> found; 
@@ -1389,8 +1405,18 @@ tstring TAppConfig::getName(const tstring& str, const tstring& context1,
       if(opt[i].name.hasPrefix(str) && (!opt[i].only_app)) 
 	found += opt[i].name;
    }
-   if(found.size()==0) 
-     userError("%sunknown option '%s'! (try --help)\n", context.c_str(), (optprefix+str).c_str());
+   if(found.size()==0)
+   {
+       if (errorIfNotFound)
+       {
+	   userError("%sunknown option '%s'! (try --help)\n", context.c_str(), (optprefix+str).c_str());
+       }
+       else
+       {
+	   userWarning("%sunknown option '%s'!\n", context.c_str(), (optprefix+str).c_str());
+	   return "";
+       }
+   }
    if(found.size() != 1) {
       tstring option = optprefix + str;
       tstring list = optprefix + join(found, (", "+optprefix));
@@ -1599,19 +1625,27 @@ bool TAppConfig::operator() (const tstring &n) const {
 }
 
 
-TACO_SET_IN TAppConfig::wasSetIn(const tstring &n) const {
+TAppConfigItem::TACO_SET_IN TAppConfig::wasSetIn(const tstring &n) const {
    if(name.contains(n)) {
       const TAppConfigItem& a(opt[name[n]]);
       return a.set_in;
    } else fatalError1("unknown parameter name '%s'\n", n.c_str());
 #ifndef __GNUC__
-   return NEVER;
+   // will never come here, avoid warning
+   return TAppConfigItem::NEVER;
 #endif
 }
 
 
+bool TAppConfig::wasSetByUser(const tstring &n) const 
+{
+    TAppConfigItem::TACO_SET_IN set = wasSetIn(n);
+    return (set == TAppConfigItem::COMMAND_LINE) || (set == TAppConfigItem::RC_FILE);
+}
+
+
 void TAppConfig::setFromStr(const tstring& str, const tstring& context, 
-			    TACO_SET_IN setin) {
+			    TAppConfigItem::TACO_SET_IN setin) {
    // prepare name n and parameter par
    tstring n;
    tstring par;
@@ -1626,7 +1660,10 @@ void TAppConfig::setFromStr(const tstring& str, const tstring& context,
    
    // set value
    if(!n.empty()) {
-      n = getName(n, context); // get name
+       // get name, ignore unknown parameters in config files with a warning
+       n = getName(n, context, "", setin != TAppConfigItem::RC_FILE); 
+       if (n.empty())
+	   return; // ignore unknown parameters (warning already printed)
       TAppConfigItem& a(opt[name[n]]);
       if(a.type == TAppConfigItem::SWITCH) { // need no param
 	 if(str.contains('=')) par = "t"; // force error
@@ -1659,7 +1696,7 @@ void TAppConfig::doEnvironVar(const char *env) {
 	printf("parsing environment variable '%s'\n", env);
       tvector<tstring> a = split(p, ",", true, true);
       for(size_t i=0; i<a.size(); i++) {
-	 setFromStr(a[i], "environment '" + tstring(env) + "'", ENVIRONMENT);
+	 setFromStr(a[i], "environment '" + tstring(env) + "'", TAppConfigItem::ENVIRONMENT);
       }
    } else {
       if(verbose_conf)
@@ -1709,7 +1746,7 @@ void TAppConfig::doRCFile(const tstring& rcfile, const tstring& clrcfile) {
    for(size_t i = 0; i < a.size(); i++) {
       if(!strchr("#;!/", a[i][0])) { // ignore comment
 	 if(!a[i].consistsOfSpace()) 
-	   setFromStr(a[i], "rc-file '" + rc_name + "' (line " + tstring(int(i + 1)) + ")", RC_FILE);
+	     setFromStr(a[i], "rc-file '" + rc_name + "' (line " + tstring(int(i + 1)) + ")", TAppConfigItem::RC_FILE);
       }
    }
 }
@@ -1763,7 +1800,8 @@ void TAppConfig::doCommandLine(int ac, char *av[], const tstring& version) {
 			}
 		     }
 		  }		     
-		  a.setValue(par, COMMAND_LINE);
+		  par.compileCString();
+		  a.setValue(par, TAppConfigItem::COMMAND_LINE);
 	       } else {
 		  userError("syntax error: missing name in long option '%s'!\n", av[i]);
 	       }
@@ -1772,8 +1810,8 @@ void TAppConfig::doCommandLine(int ac, char *av[], const tstring& version) {
 	    
 	  default:  // short name parameter
 	    for(const char *p = &av[i][1]; *p; p++) {
-	       if(char2index[*p] >= 0) { // valid short option
-		  TAppConfigItem& a(opt[char2index[*p]]);
+	       if(char2index[(unsigned char)*p] >= 0) { // valid short option
+		  TAppConfigItem& a(opt[char2index[(unsigned char)*p]]);
 		  tstring par;
 		  if(a.type != TAppConfigItem::SWITCH) { // get param
 		     if(p[1]) {
@@ -1785,7 +1823,7 @@ void TAppConfig::doCommandLine(int ac, char *av[], const tstring& version) {
 				       onlycl?"":"command line: ", p);
 		     }
 		  } 
-		  a.setValue(par, COMMAND_LINE);
+		  a.setValue(par, TAppConfigItem::COMMAND_LINE);
 		  if(a.type != TAppConfigItem::SWITCH) break;
 	       } else {
 		  userError("%sunknown option '-%c'! (try --help)\n", 
@@ -1843,19 +1881,19 @@ void TAppConfig::printValues() const {
 void TAppConfig::setComp(const tvector<tstring>& a, const tstring& context) {
    if((a.size()==1) && (a[0].consistsOfSpace())) return;
    tstring comp = a[0];
-   tstring param;
+   tstring parameter;
    if(a.size()>1) {
-      param=a[1];
-      param.unquote();
+      parameter=a[1];
+      parameter.unquote();
    }
    if(a.size()>2) {
       tstring s = join(a, "=");
       s.expandUnprintable('\'');
       fatalError2("%s: syntax error near component '%s'! (too many '=')\n", context.c_str(), s.c_str());
    }
-   if(comp=="usage") { usage = param;
-   } else if(comp=="commonheadline") { commonhead = param;
-   } else if(comp=="trailer") { trailer = param;
+   if(comp=="usage") { usage = parameter;
+   } else if(comp=="commonheadline") { commonhead = parameter;
+   } else if(comp=="trailer") { trailer = parameter;
    } else if(comp=="onlycl") { onlycl = true;
    } else if(comp=="stopat--") { stopatdd = true;
    } else if(comp=="remove--") { removedd = true;
