@@ -2,7 +2,7 @@
  * 
  * texception - basic exceptions
  * 
- * Copyright (C) 1998 by Johannes Overmann <overmann@iname.com>
+ * Copyright (C) 1998 by Johannes Overmann <Johannes.Overmann@gmx.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 #ifndef _ngw_texception_h_
 #define _ngw_texception_h_
 
+#include "config.h"
+
 extern "C" {
 #include <stdio.h>
 #include <stdarg.h>
@@ -57,7 +59,7 @@ class TException {
       va_end(ap);
    }
 #endif
-#if !(defined __USE_SVID || defined __USE_BSD || defined __USE_XOPEN_EXTENDED)
+#if !(defined HAVE_STRDUP)
    static char *strdup(const char *str) { char *buf; vasprintf(&buf, "%s", str); return buf; }
 #endif
 };
@@ -66,8 +68,8 @@ class TException {
 // general exceptions, also base classes
 class TIndexOutOfRangeException: public TException {
    TExceptionN(TIndexOutOfRangeException);
-   TIndexOutOfRangeException(int lower, int index, int upper): 
-   lower(lower), index(index), upper(upper) {}
+   TIndexOutOfRangeException(int lower_, int index_, int upper_): 
+   lower(lower_), index(index_), upper(upper_) {}
    TExceptionM3("index %d not in [%d..%d]", index, lower, upper);
    int lower, index, upper;
 };
@@ -75,18 +77,14 @@ class TIndexOutOfRangeException: public TException {
 
 class TZeroBasedIndexOutOfRangeException: public TIndexOutOfRangeException {
    TExceptionN(TZeroBasedIndexOutOfRangeException);
-   TZeroBasedIndexOutOfRangeException(int index, int total_num): TIndexOutOfRangeException(0, index, total_num-1) {}
+   TZeroBasedIndexOutOfRangeException(int index_, int total_num_): TIndexOutOfRangeException(0, index_, total_num_-1) {}
 };
 
 
 class TErrnoException: public TException {
    TExceptionN(TErrnoException);
    TErrnoException(int error = -1): err(error) { if(err < 0) err = errno; }
-#ifdef __palmos__   
-   const char *str() const { if(err >= 0) return "(strerror not implemented)"; else return "(no error)"; }
-#else
    const char *str() const { if(err >= 0) return strerror(err); else return "(no error)"; }
-#endif
    int err;
    TExceptionM2("%s (errno #%d)", str(), err);
 };
@@ -107,8 +105,8 @@ class TNotFoundException: public TException {
 
 class TFileOperationErrnoException: public TErrnoException {
    TExceptionN(TFileOperationErrnoException);
-   TFileOperationErrnoException(const char *filename_, const char *operation_, int err = -1):
-   TErrnoException(err), filename(strdup(filename_)), operation(strdup(operation_)) {}
+   TFileOperationErrnoException(const char *filename_, const char *operation_, int err_ = -1):
+   TErrnoException(err_), filename(strdup(filename_)), operation(strdup(operation_)) {}
 //   virtual ~TFileOperationErrnoException() { free(filename); free(operation); }
    const char *filename;
    const char *operation;
@@ -120,7 +118,7 @@ class TFileOperationErrnoException: public TErrnoException {
 
 class TInternalErrorException: public TException {
    TExceptionN(TInternalErrorException);
-   TInternalErrorException(const char *error = "unspecific error"): error(strdup(error)) {}
+   TInternalErrorException(const char *error_ = "unspecific error"): error(strdup(error_)) {}
 //   ~TInternalErrorException() { free(error); }
    char *error;
    TExceptionM1("internal error: %s", error);
@@ -128,7 +126,7 @@ class TInternalErrorException: public TException {
 
 class TNotInitializedException: public TInternalErrorException {
    TExceptionN(TNotInitializedException);
-   TNotInitializedException(const char *type_name): type_name(type_name) {}
+   TNotInitializedException(const char *type_name_): type_name(type_name_) {}
    TExceptionM1("internal_error: object of type '%s' is not initialized", type_name ? type_name : "<unknown>");
    const char *type_name;
 };
