@@ -2,7 +2,7 @@
  *
  * tfilesync.h - file and directory handling tolls header file
  * 
- * Copyright (C) 2000 by Johannes Overmann <overmann@iname.com>
+ * Copyright (C) 2000 by Johannes Overmann <Johannes.Overmann@gmx.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -105,7 +105,7 @@ size_t TDir::numRecursive(bool low_mem, const char *verbose, bool count_files, b
    if(verbose) { 
       verbose_num += r; 
       if(verbose_num > (old_verbose_num + COUNT_VERBOSE_STEP)) {
-	 printf("counting %s in %s: %11d\r", count_files ? (count_dirs ? "files and dirs" : "files") : (count_dirs ? "dirs" : "nothing"), verbose, verbose_num);
+	 printf("counting %s in %s: %11u\r", count_files ? (count_dirs ? "files and dirs" : "files") : (count_dirs ? "dirs" : "nothing"), verbose, unsigned(verbose_num));
 	 fflush(stdout); 
 	 old_verbose_num = verbose_num;
       }
@@ -118,8 +118,15 @@ size_t TDir::numRecursive(bool low_mem, const char *verbose, bool count_files, b
 
 void TDir::scan() const {
    if(scanned) return;
+   assert(subTreeContext);
    if(name().empty()) throw TNotInitializedException("TDir");
    TDir *t = const_cast<TDir*>(this);
+   // maximum depth already reached? if so treat directory as empty
+   if(depth >= subTreeContext->max_depth)
+   {
+       t->scanned = true;
+       return;
+   }
    // prevent crossing filesystems?
    if(!subTreeContext->cross_filesystems) {
       if(subTreeContext->root->device() != device()) {
@@ -138,7 +145,7 @@ void TDir::scan() const {
 	 TFile f((name() + "/") + dire->d_name);
 	 if((dirs_left || no_leaf_optimize) && f.isdir()) {
 	    t->name2dirid[dire->d_name] = dirs.size();
-	    t->dirs[dirs.size()] = TDir(f, subTreeContext);
+	    t->dirs[dirs.size()] = TDir(f, *subTreeContext, depth + 1);
 	    dirs_left--;
 	 } else {
 	    t->name2fileid[dire->d_name] = files.size();
@@ -189,7 +196,6 @@ tvector<tstring> filterExtensions(const tvector<tstring>& list, const tvector<ts
 
    // filter list
    tvector<tstring> r;
-   tstring e;
    for(size_t j = 0; j < list.size(); j++) {
       tstring e = list[j];
       e.extractFilenameExtension();
