@@ -2,7 +2,7 @@
  *
  * tfilesync.h - file and directory handling tolls header file
  * 
- * Copyright (C) 2000 by Johannes Overmann <overmann@iname.com>
+ * Copyright (C) 2000 by Johannes Overmann <Johannes.Overmann@gmx.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <assert.h>
 #include "tstring.h"
 #include "texception.h"
 #include "tmap.h"
@@ -45,7 +46,11 @@
 // own device type which is a simple 64 bit unsigned integer
 typedef unsigned long long mydev_t;
 inline mydev_t dev_t2mydev_t(const dev_t& dev) {
+#ifdef __APPLE__
+   return *(reinterpret_cast<const unsigned int*>(&dev));
+#else
    return *(reinterpret_cast<const mydev_t*>(&dev));
+#endif
 }
 inline dev_t mydev_t2dev_t(mydev_t dev) {
    return *(reinterpret_cast<const dev_t*>(&dev));
@@ -72,7 +77,7 @@ inline bool operator != (const TFileInstance& d1, const TFileInstance& d2) {
 class TFile {
  public:
    // cons & des
-   TFile(const tstring& name=tstring()): name_(name), stated(false) {}
+   TFile(const tstring& fname = tstring()): name_(fname), stated(false) {}
    ~TFile() {}
    void invalidateStat() const { const_cast<TFile*>(this)->stated = false; }
    
@@ -138,29 +143,30 @@ class TDir;
 
 class TSubTreeContext {
  public:
-   TSubTreeContext(bool _cross_filesystems): root(0), cross_filesystems(_cross_filesystems) {}
+   TSubTreeContext(bool cross_filesystems, size_t max_depth = 0xffffffff): root(0), cross_filesystems(cross_filesystems), max_depth(max_depth) {}
    TDir *root;
    bool cross_filesystems;
+   size_t max_depth;
 };
 
 
 class TDir: public TFile {
  public:
-   TDir(): TFile(), scanned(false), subTreeContext(0) {}
-   TDir(const tstring& name, TSubTreeContext& subTreeContext_): TFile(name), scanned(false), subTreeContext(&subTreeContext_) { init(); }
-   TDir(const TFile& file, TSubTreeContext *subTreeContext_): TFile(file), scanned(false), subTreeContext(subTreeContext_) { init(); }
+   TDir(): TFile(), scanned(false), subTreeContext(0), depth(0) {}
+   TDir(const tstring& fname, TSubTreeContext& subTreeContext_, size_t depth = 0): TFile(fname), scanned(false), subTreeContext(&subTreeContext_), depth(depth) { init(); }
+   TDir(const TFile& file_, TSubTreeContext& subTreeContext_, size_t depth = 0): TFile(file_), scanned(false), subTreeContext(&subTreeContext_), depth(depth) { init(); }
    ~TDir() {}
 
    // read only interface
-   const TFile& file(const tstring& name) const { scan(); if(name2fileid.contains(name)) return files[name2fileid[name]]; else throw TNotFoundException(); }
-   const TDir & dir (const tstring& name) const { scan(); if(name2dirid .contains(name)) return dirs [name2dirid [name]]; else throw TNotFoundException(); }
+   const TFile& file(const tstring& fname) const { scan(); if(name2fileid.contains(fname)) return files[name2fileid[fname]]; else throw TNotFoundException(); }
+   const TDir & dir (const tstring& fname) const { scan(); if(name2dirid .contains(fname)) return dirs [name2dirid [fname]]; else throw TNotFoundException(); }
    const TFile& file(size_t index) const { scan(); if(index >= files.size()) throw TZeroBasedIndexOutOfRangeException(index, files.size()); return files[index];}
    const TDir & dir (size_t index) const { scan(); if(index >= dirs .size()) throw TZeroBasedIndexOutOfRangeException(index, dirs .size()); return dirs [index];}
    size_t numFiles() const { scan(); return files.size();}
    size_t numDirs () const { scan(); return dirs .size();}
-   bool containsFile(const tstring& name) const { scan(); return name2fileid.contains(name); }
-   bool containsDir (const tstring& name) const { scan(); return name2dirid .contains(name); }
-   bool contains    (const tstring& name) const { return containsDir(name) || containsFile(name); }
+   bool containsFile(const tstring& fname) const { scan(); return name2fileid.contains(fname); }
+   bool containsDir (const tstring& fname) const { scan(); return name2dirid .contains(fname); }
+   bool contains    (const tstring& fname) const { return containsDir(fname) || containsFile(fname); }
    bool isEmpty() const { return dirs.empty() && files.empty(); }
    void invalidateContents() { freeMem(); }
    size_t numRecursive(bool low_mem = true, const char *verbose = 0, bool count_files = true, bool count_dirs = true) const;
@@ -173,11 +179,10 @@ class TDir: public TFile {
 #endif      
    }
    
-   
  private:
    // private helpers
    void scan() const;
-   void init() { if(subTreeContext->root == 0) subTreeContext->root = this; }
+   void init() { assert(subTreeContext); if(subTreeContext->root == 0) subTreeContext->root = this; }
    
    // private data
    bool scanned;
@@ -186,6 +191,7 @@ class TDir: public TFile {
    tmap<tstring,int> name2dirid;
    tmap<int,TDir> dirs;
    TSubTreeContext *subTreeContext;
+   size_t depth;
    
    // private static data
    static size_t verbose_num;
