@@ -2,7 +2,7 @@
  *
  * NUL byte safe string implementation
  * 
- * Copyright (C) 1997-2001 by Johannes Overmann <overmann@iname.com>
+ * Copyright (C) 1997-2001 by Johannes Overmann <Johannes.Overmann@gmx.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,13 +19,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * *GPL*END*/  
 
-#ifdef __palmos__
-#include "tpalmlib.h"
-#else
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#endif
 #include <errno.h>
 #include "tstring.h"
 #include "texception.h"
@@ -87,14 +83,18 @@
 // 2002:
 // 22:25 08 Apr expandUnpritable: allow high ISO graphical characters (ASCII 161-255), better nul_mem and zero_mem sizes for 64 bit systems
 
+// 2003:
+// 22:20 27 Jan length of nul_mem and zero_mem fixed
 
+// 2006:
+// 27 Jul: palmos support removed
 
 
 // global static null and zero rep members
 tstring::Rep* tstring::Rep::nul = 0;
-int tstring::Rep::nul_mem[(sizeof(Rep)/sizeof(int) + 1) * 2];
+char tstring::Rep::nul_mem[sizeof(Rep) + 1];
 tstring::Rep* tstring::Rep::zero = 0;
-int tstring::Rep::zero_mem[(sizeof(Rep)/sizeof(int) + 1) * 2];
+char tstring::Rep::zero_mem[sizeof(Rep) + 2];
 
 
 // non inline Rep implementations
@@ -120,7 +120,7 @@ tstring::Rep *tstring::Rep::create(size_t tmem) {
 void tstring::Rep::createNulRep() {
    nul = (Rep *)nul_mem;
    nul->len = 0;
-   nul->mem = 1;
+   nul->mem = 0;
    nul->ref = 1; // never modify/delete static object
    nul->vulnerable = false;
    nul->terminate();
@@ -142,19 +142,19 @@ void tstring::Rep::createZeroRep() {
 
 tstring::tstring(const char *s):rep(0) {
    if(s){
-      int len = strlen(s);
-      rep = Rep::create(len);
-      rep->len = len;
+      int l = strlen(s);
+      rep = Rep::create(l);
+      rep->len = l;
       strcpy(rep->data(), s);
    } else rep = Rep::nulRep()->grab();
 }
 
 
-tstring::tstring(const char *s, size_t len):rep(0) {
-   if(s && (len > 0)) {
-      rep = Rep::create(len);
-      rep->len = len;
-      memcpy(rep->data(), s, len);      
+tstring::tstring(const char *s, size_t l):rep(0) {
+   if(s && (l > 0)) {
+      rep = Rep::create(l);
+      rep->len = l;
+      memcpy(rep->data(), s, l);      
       rep->terminate();
    } else rep = Rep::nulRep()->grab();
 }
@@ -286,7 +286,6 @@ void tstring::detachResize(size_t minsize) {
 /// detach from string pool and declare that string might be externally modified (the string has become vulnerable)
 void tstring::invulnerableDetach() { detach(); rep->vulnerable = true; }
 
-#ifndef __palmos__
 /// check for 0 in string (then its not a real cstring anymore)
 bool tstring::containsNulChar() const {
    rep->terminate();
@@ -301,28 +300,28 @@ bool tstring::containsNulChar() const {
 const char *tstring::pSuf(size_t max) const {
    return rep->data()+((max>=rep->len)?0:(rep->len-max));
 }
-#endif
+
 
 /// sprintf into this string
 void tstring::sprintf(const char *format, ...) {
    va_list ap;
    int ret = -1;
    va_start(ap, format);
-#if defined(__STRICT_ANSI__) || defined(__palmos__)
+#if defined(__STRICT_ANSI__)
    // this is the unsecure and dirty but ansi compatible version
    detachResize(256);      
    ret = vsprintf(rep->data(), format, ap); // not secure! may write out of bounds!
 #else
    // this is the clean version (never overflows)
-   int size = 16/4;
+   int s = 16/4;
    do { 
-      if(ret <= size)
-	size <<= 2; // fast increase, printf may be slow
+      if(ret <= s)
+	s <<= 2; // fast increase, printf may be slow
       else 
-	size = ret + 8; // C99 standard, after first iteration this should be large enough
-      detachResize(size);
-      ret = vsnprintf(rep->data(), size, format, ap); 
-   } while((ret == -1) || (ret >= size));
+	s = ret + 8; // C99 standard, after first iteration this should be large enough
+      detachResize(s);
+      ret = vsnprintf(rep->data(), s, format, ap); 
+   } while((ret == -1) || (ret >= s));
 #endif
    va_end(ap);
    rep->len = ret;
@@ -432,7 +431,6 @@ tstring tstring::scanToken(size_t& scanner, int flags,
 }
 
 
-#ifndef __palmos__
 tstring tstring::shortFilename(size_t maxchar) const {
    if(rep->len <= maxchar) return *this;
    if(maxchar < 3) return "";
@@ -521,8 +519,6 @@ double tstring::binaryPercentage() const {
 }
 
 
-#endif
-
 bool tstring::isLower() const {
    if(rep->len == 0) return false;
    for(size_t i = 0; i < rep->len; i++) 
@@ -601,24 +597,19 @@ static const char *bytesearch(const char *mem, int mlen,
 }
 
 
-#ifndef __palmos__
-int tstring::searchReplace(const tstring& tsearch, const tstring& replace, 
+int tstring::searchReplace(const tstring& tsearch, const tstring& replace_, 
 			   bool ignore_case, bool whole_words,
 			   bool preserve_case, int progress,
 			   const tstring& pre_padstring, const tstring& post_padstring, tvector<int> *match_pos, int max_num) {
    // get new length and positions
-#ifndef __palmos__
    if(progress) { putc('S', stderr);fflush(stderr); }
-#endif   
    int num = search(tsearch, ignore_case, whole_words, progress);
-#ifndef __palmos__
    if(progress) { putc('R', stderr);fflush(stderr); }   
-#endif   
    if(num==0) {
       return 0;
    }
    if(num >= max_num) num = max_num;
-   int newlen = rep->len + num*(replace.rep->len-tsearch.rep->len + 
+   int newlen = rep->len + num*(replace_.rep->len-tsearch.rep->len + 
 				pre_padstring.len()+post_padstring.len());
 
    // create new string 
@@ -628,9 +619,7 @@ int tstring::searchReplace(const tstring& tsearch, const tstring& replace,
    const char *r;                // found substring
    int mlen = rep->len;          // rest of read mem
    for(int i=0; i < num; i++) {
-#ifndef __palmos__
       if(progress>0) if((i%progress)==0) {putc('.', stderr);fflush(stderr);}
-#endif   
       r = bytesearch(p, mlen, tsearch.rep->data(), tsearch.rep->len, ignore_case, whole_words);
       memcpy(q, p, r-p); // add skipped part
       q += r-p;      
@@ -638,12 +627,12 @@ int tstring::searchReplace(const tstring& tsearch, const tstring& replace,
       memcpy(q, pre_padstring.rep->data(), pre_padstring.rep->len); // add pre pad
       q += pre_padstring.len();
       if(!preserve_case) { // add replaced part
-	 memcpy(q, replace.rep->data(), replace.rep->len);
+	 memcpy(q, replace_.rep->data(), replace_.rep->len);
       } else {
-	 tstring rep(preserveCase(tstring(r, tsearch.rep->len), replace.rep->data()));
-	 memcpy(q, rep.rep->data(), rep.rep->len);
+	 tstring rr(preserveCase(tstring(r, tsearch.rep->len), replace_.rep->data()));
+	 memcpy(q, rr.rep->data(), rr.rep->len);
       }
-      q += replace.rep->len;      
+      q += replace_.rep->len;      
       memcpy(q, post_padstring.rep->data(), post_padstring.rep->len); // add post pad
       q += post_padstring.len();
       if(match_pos) (*match_pos) += int(q-newrep->data()); // enter end
@@ -658,7 +647,6 @@ int tstring::searchReplace(const tstring& tsearch, const tstring& replace,
    return num;
 }
 
-#endif
 
 int tstring::search(const tstring& pat, bool ignore_case, bool whole_words, int progress, tvector<int> *match_pos) const {
    if(pat.empty()) return -1;
@@ -672,14 +660,30 @@ int tstring::search(const tstring& pat, bool ignore_case, bool whole_words, int
       mlen -= pat.rep->len;
       p = q + pat.rep->len;
       if(match_pos) (*match_pos) += int(p-rep->data());
-#ifndef __palmos__
       if(progress>0) if((num%progress)==0) {putc('.', stderr);fflush(stderr);}
-#endif   
    }
    return num;
 }
 
 
+/// replace substring
+void tstring::replace(size_t start, size_t len_, const tstring &str) {
+   if(start > length()) return;
+   if(start + len_ > length()) return;
+   if(str.length() > len_)
+     detachResize(length() + str.length() - len_);
+   else
+     detach();
+   if(str.length() != len_)
+     memmove(rep->data() + start + str.length(), rep->data() + start + len_, length() - start - len_);
+   // insert string
+   memcpy(rep->data() + start, str.data(), str.length());
+   // fix length
+   rep->len += str.length() - len_;
+   rep->terminate();
+}
+
+
 bool tstring::hasPrefix(const tstring& pref) const {
    if(pref.rep->len > rep->len) return false;
    return memcmp(rep->data(), pref.rep->data(), pref.rep->len)==0;
@@ -712,7 +716,7 @@ void tstring::truncate(size_t max) {
 
 void tstring::replaceUnprintable(bool only_ascii) {
    for(size_t i = 0; i < rep->len; i++) {
-      unsigned char& c = (unsigned char)(*rep)[i];
+      unsigned char& c = (unsigned char &)(*rep)[i];
       if(!isprint(c)) {
 	 if(c < ' ') {
 	    c = '!';
@@ -760,7 +764,7 @@ void tstring::unquote(bool allow_bslash, bool crop_space) {
    rep->len = strlen(rep->data());   
 }
 
-#ifndef __palmos__
+
 tstring tstring::getFitWordsBlock(size_t max) {
    tstring r = getFitWords(max);
    size_t spaces;
@@ -992,10 +996,10 @@ void tstring::compileCString() {
 	    c = '\v';
 	    break;
 	  case 'x': // hex
-	    char *q;
-	    c = strtol(p, &q, 16);
-	    i += q-p;
-	    p = q;
+	    char *qq;
+	    c = strtol(p, &qq, 16);
+	    i += qq-p;
+	    p = qq;
 	    break;	    
 	  case '0': // octal
 	  case '1':
@@ -1056,7 +1060,7 @@ void tstring::removeHTMLTags(int& level) {
    rep->len = l;
    rep->terminate();
 }
-#endif
+
 
 void tstring::cropSpace(void) {
    size_t first = rep->len;
@@ -1234,7 +1238,6 @@ tstring preserveCase(const tstring& from, const tstring& to) {
 }
 
 
-#ifndef __palmos__
 const char *progressBar(const char *message, unsigned int n, unsigned int max, int width) {
    // max size of a buffer
 #define size 1024
@@ -1299,7 +1302,7 @@ bool tstring::readLine(FILE *file) {
 
 
 size_t tstring::write(FILE *file) const {
-   return fwrite(rep->data(), 1, rep->len, file);   
+   return fwrite(rep->data(), 1, rep->len, file);
 }
 
 
@@ -1318,7 +1321,7 @@ int tstring::readFile(const char *filename) {
 
    if(stat(filename, &buf)) return -1; // does not exist
    FILE *f=fopen(filename, "rb");
-   if(f==0) return -2;                 // no permission?
+   if(f == 0) return -2;                 // no permission?
    int r = read(f, buf.st_size);
    fclose(f);
    if(r != buf.st_size) return -3;     // read error
@@ -1326,6 +1329,16 @@ int tstring::readFile(const char *filename) {
 }
 
 
+int tstring::writeFile(const char *filename) {
+   FILE *f = fopen(filename, "wb");
+   if(f == 0) return -2;                 // no permission?
+   int r = write(f);
+   fclose(f);
+   if(r != int(length())) return -3;     // write error
+   return 0;
+}
+
+
 tvector<tstring> loadTextFile(const char *fname) {
    FILE *f = fopen(fname, "r");
    if(f==0) throw TFileOperationErrnoException(fname, "fopen(mode='r')", errno);
@@ -1343,5 +1356,4 @@ tvector<tstring> loadTextFile(FILE *file) {
    r.pop_back();
    return r;
 }
-#endif /* __palmos__ */
 
